package io.milvus.response;

import io.milvus.grpc.GetImportStateResponse;
import io.milvus.grpc.ImportState;
import io.milvus.grpc.KeyValuePair;
import io.milvus.param.Constant;
import lombok.NonNull;

import java.util.List;

/**
 * Util class to wrap response of <code>getBulkloadState</code> interface.
 */
public class GetBulkloadStateWrapper {
    private final GetImportStateResponse response;

    public GetBulkloadStateWrapper(@NonNull GetImportStateResponse response) {
        this.response = response;
    }

    /**
     * Gets ID of the bulk load task.
     *
     * @return Long ID of the bulk load task
     */
    public long getTaskID() {
        return response.getId();
    }

    /**
     * Gets the long ID array for auto-id primary key, generated by bulk load task.
     *
     * @return List&lt;Long&gt; ID array returned by bulk load task
     */
    public List<Long> getAutoGeneratedIDs() {
        return response.getIdListList();
    }

    /**
     * Gets state of the bulk load task.
     *
     * @return ImportState state of the bulk load task
     */
    public ImportState getState() {
        return response.getState();
    }

    /**
     * Gets how many rows were imported by the bulk load task.
     *
     * @return Long how many rows were imported by the bulk load task
     */
    public long getImportedCount() {
        return response.getRowCount();
    }

    /**
     * Gets failed reason of the bulk load task.
     *
     * @return String failed reason of the bulk load task
     */
    public String getFailedReason() {
        return getInfo(Constant.FAILED_REASON);
    }

    /**
     * Gets target files of the bulk load task.
     *
     * @return String target files of the bulk load task
     */
    public String getFiles() {
        return getInfo(Constant.IMPORT_FILES);
    }

    /**
     * A flag indicating (heuristically) whether import data are queryable (i.e. loaded in query nodes).
     *
     * @return boolean whether import data are queryable(heuristically)
     */
    public boolean HeuristicQueryable() {
        return response.getHeuristicDataQueryable();
    }

    /**
     * A flag indicating (heuristically) whether import data are indexed.
     *
     * @return boolean whether import data are queryable(heuristically)
     */
    public boolean HeuristicIndexed() {
        return response.getHeuristicDataQueryable();
    }

    private String getInfo(@NonNull String key) {
        List<KeyValuePair> infos = response.getInfosList();
        for (KeyValuePair kv : infos) {
            if (kv.getKey().compareTo(key) == 0) {
                return kv.getValue();
            }
        }

        return "";
    }

    /**
     * Construct a <code>String</code> by {@link DescCollResponseWrapper} instance.
     *
     * @return <code>String</code>
     */
    @Override
    public String toString() {
        return "bulk load task state{" +
                ", autoGeneratedIDs:" + getAutoGeneratedIDs() +
                ", state:" + getState().name() +
                ", failed reason:" + getFailedReason() +
                ", files:" + getFiles() +
                '}';
    }
}
